from pygraham import *


def test_list():
	v = list([1, 2, 3])
	v = v.map(lambda x: 2 * x)

	print(v)

def test_list_map_reduce():
	v = list([1, 2, 3])
	v = v.map(lambda x: 2 * x).reduce(lambda x,y: x+y)

	print(v)

def test_list_map_filter_reduce():
	v = list([1, 2, 3])
	v = v.map(lambda x: 2 * x).filter(lambda x: (x%2==0)).reduce(lambda x,y: x+y)

	print(v)

def test_list_map_filter_zip():
	v = list([1, 2, 3])
	v = v.map(lambda x: 2 * x).filter(lambda x: (x%2==0)).zip([2,3,4], lambda x, y: (x+y, x*y))

	print(v)

def test_dict():
	v = dict({9:2,11:3,33:4})
	v = v.map(lambda x: 2 * x[0] + 2*x[1])

	print(v)

def test_dict_unzip():
	v = dict({9:2,11:3,33:4})
	u, v = v.unzip(lambda x: x[0], lambda x: x[1])

	print(u)
	print(v)

def test_dict_unzip_zip():
	v = dict({9:2,11:3,33:4})
	u, v = v.unzip(lambda x: x[0], lambda x: x[1])
	u = u.zip(v,lambda x,y: (x,y))

	print(u)

def test_dict_unzip_zip_to_dict():
	v = dict({9:2,11:3,33:4})
	u, v = v.unzip(lambda x: x[0], lambda x: x[1])
	u = u.zip(v,lambda x,y: (x,y))
	u = dict(u)

	print(u)

def test_set():
	v = set([4, 43, 36])
	v = v.map(lambda x: 2 * x)

	print(v)


if __name__ == "__main__":
	print("test_list()")
	test_list()
	print("-------------------------")
	print("test_list_map_reduce()")
	test_list_map_reduce()
	print("-------------------------")
	print("test_list_map_filter_reduce()")
	test_list_map_filter_reduce()
	print("-------------------------")
	print("test_list_map_filter_zip()")
	test_list_map_filter_zip()
	print("-------------------------")
	print("test_dict()")
	test_dict()
	print("-------------------------")
	print("test_dict_unzip()")
	test_dict_unzip()
	print("-------------------------")
	print("test_dict_unzip_zip()")
	test_dict_unzip_zip()
	print("-------------------------")
	print("test_dict_unzip_zip_to_dict()")
	test_dict_unzip_zip_to_dict()
	print("-------------------------")
	print("test_set()")
	test_set()
	print("-------------------------")



def map(o, l):
	tmp = o.__iter__()
	new = list([])
	for i in tmp:
		new += [l(i)]
	return new


def reduce(o, l):
	tmp = o.__iter__()
	r = 0
	for i in tmp:
		r = l(r, i)
	return r


def filter(o, l):
	tmp = o.__iter__()
	new = list([])
	for i in tmp:
		if l(i):
			new += [i]
	return new


def zip(o, v, l):
	v = list(v)
	new = list([])
	for i in range(len(o)):
		new += [l(o[i], v[i])]
	return new

def next(o, l):
	for i in range(len(o)):
		l(o[i])
	return o


def unzip(o, sx, dx):
	tmp = o.__iter__()
	new_sx = list([])
	new_dx = list([])
	for i in tmp:
		new_sx += [sx(i)]
		new_dx += [dx(i)]
	return new_sx, new_dx


def sort(o, l): # with respect to a rule
	pass

def show(o, l):
	tmp = o.__iter__()
	for i in tmp:
		print(i, end=' ')
	print()
	return o

class list(list):
	def map(self, l):
		return map(self, l)

	def reduce(self, l):
		return reduce(self, l)

	def filter(self, l):
		return filter(self, l)

	def unzip(self, sx, dx):
		return unzip(list(self), sx, dx)

	def zip(self, v, l):
		return zip(list(self), v, l)

	def show(self, l):
		return show(list(self), l)

class dict(dict):
	def map(self, l):
		return map(list(self.items()), l)

	def reduce(self, l):
		return reduce(list(self.items()), l)

	def filter(self, l):
		return filter(list(self.items()), l)

	def unzip(self, sx, dx):
		return unzip(list(self.items()), sx, dx)

	def zip(self, v, l):
		return zip(list(self), v, l)

	def show(self, l):
		return show(list(self), l)

class set(set):
	def map(self, l):
		return map(list(self), l)

	def reduce(self, l):
		return reduce(list(self), l)

	def filter(self, l):
		return filter(list(self), l)

	def unzip(self, sx, dx):
		return unzip(list(self), sx, dx)

	def zip(self, v, l):
		return zip(list(self), v, l)

	def show(self, l):
		return show(list(self), l)
